@@ -6,9 +6,7 @@ use warnings;
 
 use List::Util qw{ first };
 use List::MoreUtils qw{ true first_index };
-
-our $VERSION = '0.40';
-
+use Data::Dump 'dd';
 
 
 my %role_type = (
@@ -24,38 +22,28 @@ my %role_type = (
 sub new {
     my $class = shift;
 
-    my $self = $class->SUPER::new( shift );
+    our $self = $class->SUPER::new( shift );
 
-    $self->{latlon}         = {};
+    $self->{nod}            = {};
     $self->{waychain}       = {};
     $self->{mpoly}          = {};
     $self->{ways_to_load}   = {};
 
     our %param = @_;
 
-    
-    
-    ## First pass - load nodes and multipolygon parameters
+    ## First pass - load multipolygon parameters
 
     my $osm_pass1 = sub {
         my ($obj) = @_;
 
-        # node
-        if ( $obj->{type} eq 'node' ) {
-            $self->{latlon}->{ $obj->{id} } = pack 'Z*Z*', $obj->{lat}, $obj->{lon};
-        }
-
-        #multipolygon
-        if ( $obj->{type} eq 'relation' 
-                && exists $obj->{tag}->{type}
-                && $obj->{tag}->{type} =~ /multipolygon|boundary/ ) {
-            # old-style - list of inner rings
+        if ( $obj->{tag}->{type} =~ /multipolygon|boundary/ ) {
+            # old-style multipolygons - load lists of inner rings
             if ( (true { $_->{role} eq 'outer' } @{ $obj->{members} }) == 1 ) {
                 my $outer = first { $_->{role} eq 'outer' } @{ $obj->{members} };
                 $self->{mpoly}->{$outer->{ref}} = 
                     [ map { $_->{ref} } grep { $_->{role} eq 'inner' } @{ $obj->{members} } ];
             }
-            # advanced - ways
+            # advanced multipolygons - load lists of ways
             for my $member ( @{ $obj->{members} } ) {
                 next unless $member->{type} eq 'way';
                 next unless exists $role_type{ $member->{role} };
@@ -66,30 +54,29 @@ sub new {
         &{ $param{pass1} }( $obj )  if exists $param{pass1};
     };
 
-    $self->SUPER::parse( $osm_pass1, only => [ 'node', 'relation' ] );
+    $self->SUPER::parse( $osm_pass1, only => 'relation' );
 
-
-    
-    ## Second pass - load necessary ways
+    ## Second pass - load necessary primitives
 
     $self->seek_to(0);
-    # $self->seek_to_ways();
 
-    &{ $param{between} }( $self )  if exists $param{between};
+    &{ $param{between} }()  if exists $param{between};
 
     my $osm_pass2 = sub {
-        my ($obj, $self) = @_;
+        my ($obj) = @_;
 
-        if ( $obj->{type} eq 'way' && exists $self->{ways_to_load}->{$obj->{id}} ) {
+        if ( $obj->{type} eq 'node' ) {
+            $self->{nod}->{ $obj->{id} } = [ $obj->{lon}, $obj->{lat} ];
+        }
+        elsif ( $obj->{type} eq 'way' && exists $self->{ways_to_load}->{$obj->{id}} ) {
             $self->{waychain}->{ $obj->{id} } = $obj->{chain};
             delete $self->{ways_to_load}->{$obj->{id}};
         }
 
-        &{ $param{pass2} }( $obj, $self )  if exists $param{pass2};
+        &{ $param{pass2} }( $obj )  if exists $param{pass2};
     };
 
-    #$self->SUPER::parse( $osm_pass2 );
-    $self->parse( $osm_pass2 );
+    $self->SUPER::parse( $osm_pass2 );
 
     $self->seek_to(0);
 
@@ -100,27 +87,26 @@ sub new {
 
 sub parse {
 
-    my $self = shift;
+    our $self = shift;
     our $callback = shift;
 
     my $parse_extent = sub {
         my ($obj) = @_;
 
-        # closed ways and old-style multipolygons
-        if ( $obj->{type} eq 'way' && $obj->{chain}->[0] eq $obj->{chain}->[-1] ) {
-            # $obj->{outer} = [ [ @{$obj->{chain}} ] ];
-            $obj->{outer} = [ $obj->{chain} ];
-            if ( exists $self->{mpoly}->{ $obj->{id} } ) {
-                for my $inner ( @{ $self->{mpoly}->{ $obj->{id} } } ) {
-                    next unless exists $self->{waychain}->{$inner};
-                    next unless $self->{waychain}->{$inner}->[0] eq $self->{waychain}->{$inner}->[-1];
-                    push @{ $obj->{inner} }, $self->{waychain}->{$inner};
-                }
+        # old-style multipolygons
+        if ( $obj->{type} eq 'way' && exists $self->{mpoly}->{ $obj->{id} }
+                && $obj->{chain}->[0] eq $obj->{chain}->[-1] ) {
+            $obj->{outer} = [ [ @{$obj->{chain}} ] ];
+            # $obj->{outer} = [ $obj->{chain} ];
+            for my $inner ( @{ $self->{mpoly}->{ $obj->{id} } } ) {
+                next unless exists $self->{waychain}->{$inner};
+                next unless $self->{waychain}->{$inner}->[0] eq $self->{waychain}->{$inner}->[-1];
+                push @{ $obj->{inner} }, $self->{waychain}->{$inner};
             }
         }
 
         # advanced multipolygons
-        if ( $obj->{type} eq 'relation' && exists $obj->{tag}->{type} && $obj->{tag}->{type} =~ /multipolygon|boundary/ ) {
+        if ( $obj->{type} eq 'relation' && $obj->{tag}->{type} =~ /multipolygon|boundary/ ) {
 
             for my $contour_type ( 'outer', 'inner' ) {
     
@@ -170,21 +156,12 @@ sub parse {
             } # outers/inners
         } # advanced multipolygon
 
-        &$callback( $obj, $self );
+        &$callback( @_ );
     };
     
     $self->SUPER::parse( $parse_extent, @_ );    
 }
 
-sub latlon {
-    my $self = shift;
-    my ($node_id) = @_;
-
-    return exists $self->{latlon}->{$node_id}
-        ? ( unpack 'Z*Z*', $self->{latlon}->{$node_id} )
-        : undef;
-}
-
 
 1;
 
@@ -211,14 +188,14 @@ Geo::Parse::OSM::Multipass extends Geo::Parse::OSM class to resolve geometry.
     my $osm = Geo::Parse::OSM::Multipass->new( 'planet.osm' );
 
 Creates parser instance and makes two passes:
-1 - load node coordinates and create list of multipolygon parts
-2 - load those parts
+1 - only relations, create the list of multipolygon parts
+2 - load those parts and nodes
 
 You can add extra custom callback function:
 
     my $osm = Geo::Parse::OSM::Multipass->new( 'planet.osm', pass1 => sub{ ... } );
 
-* pass1 - is called for every object during 1st pass
+* pass1 - for every object during 1st pass
 * pass2 - same for second pass
 * between - before 2nd pass
 
@@ -229,13 +206,6 @@ Same as in Geo::Parse::OSM, but callback object has additional fields for multip
 * outer - list of outer rings (ring is a closed list of node ids)
 * inner - inner rings
 
-=head2 latlon
-
-Returns coordinates of node (after 2nd pass)
-
-    my ($lat,$lon) = $osm->latlon( '1234578' );
-
-
 =head1 AUTHOR
 
 liosha, C<< <liosha at cpan.org> >>
